@@ -21,6 +21,10 @@ else
     MISTRAL_BIN_DIR=$(get_python_exec_prefix)
 fi
 
+# By default, mistral is installed in DEVSTACK_VENV
+# NOTE(arnaud) maybe check if PROJECT_VENV should be used here
+MISTRAL_ENV_DIR=${DEVSTACK_VENV}
+
 # Toggle for deploying Mistral API under HTTPD + mod_wsgi
 MISTRAL_USE_MOD_WSGI=${MISTRAL_USE_MOD_WSGI:-True}
 
@@ -60,10 +64,6 @@ function mkdir_chown_stack {
 # configure_mistral - Set config files, create data dirs, etc
 function configure_mistral {
 
-    # create and clean up auth cache dir
-    mkdir_chown_stack "$MISTRAL_AUTH_CACHE_DIR"
-    rm -f "$MISTRAL_AUTH_CACHE_DIR"/*
-
     mkdir_chown_stack "$MISTRAL_CONF_DIR"
 
     # Generate Mistral configuration file and configure common parameters.
@@ -77,7 +77,7 @@ function configure_mistral {
     #-------------------------
 
     # Setup keystone_authtoken section
-    configure_auth_token_middleware $MISTRAL_CONF_FILE mistral $MISTRAL_AUTH_CACHE_DIR
+    configure_keystone_authtoken_middleware $MISTRAL_CONF_FILE mistral
     iniset $MISTRAL_CONF_FILE keystone_authtoken www_authenticate_uri $KEYSTONE_AUTH_URI_V3
 
     # Setup RabbitMQ credentials
@@ -109,7 +109,7 @@ function configure_mistral {
     if [[ ! -z "$MISTRAL_COORDINATION_URL" ]]; then
         iniset $MISTRAL_CONF_FILE coordination backend_url "$MISTRAL_COORDINATION_URL"
     elif is_service_enabled etcd3; then
-        iniset $MISTRAL_CONF_FILE coordination backend_url "etcd3+http://${SERVICE_HOST}:$ETCD_PORT"
+        iniset $MISTRAL_CONF_FILE coordination backend_url "etcd3+http://${SERVICE_HOST}:$ETCD_PORT?api_version=v3"
     fi
 }
 
@@ -126,17 +126,6 @@ function init_mistral {
 function install_mistral {
     setup_develop $MISTRAL_DIR
 
-    # installing python-nose.
-    if is_fedora; then
-	if python3_enabled; then
-            real_install_package python3-nose
-	else
-            real_install_package python2-nose
-        fi
-    else
-        real_install_package python-nose
-    fi
-
     if is_service_enabled horizon; then
         _install_mistraldashboard
     fi
@@ -245,23 +234,16 @@ function _mistral_cleanup_apache_wsgi {
 function _config_mistral_apache_wsgi {
     local mistral_apache_conf
     mistral_apache_conf=$(apache_site_config_for mistral-api)
-    local mistral_ssl=""
-    local mistral_certfile=""
-    local mistral_keyfile=""
     local mistral_api_port=$MISTRAL_SERVICE_PORT
-    local venv_path=""
 
     sudo cp $MISTRAL_FILES_DIR/apache-mistral-api.template $mistral_apache_conf
     sudo sed -e "
         s|%PUBLICPORT%|$mistral_api_port|g;
         s|%APACHE_NAME%|$APACHE_NAME|g;
-        s|%MISTRAL_BIN_DIR%|$MISTRAL_BIN_DIR|g;
+        s|%MISTRAL_DIR%|$MISTRAL_DIR|g;
+        s|%MISTRAL_ENV_DIR%|$MISTRAL_ENV_DIR|g;
         s|%API_WORKERS%|$API_WORKERS|g;
-        s|%SSLENGINE%|$mistral_ssl|g;
-        s|%SSLCERTFILE%|$mistral_certfile|g;
-        s|%SSLKEYFILE%|$mistral_keyfile|g;
         s|%USER%|$STACK_USER|g;
-        s|%VIRTUALENV%|$venv_path|g
     " -i $mistral_apache_conf
 }
 
