@@ -25,12 +25,6 @@ eventlet.monkey_patch(
     socket=True,
     thread=False if '--use-debugger' in sys.argv else True,
     time=True)
-# Monkey patch the original current_thread to use the up-to-date _active
-# global variable. See https://bugs.launchpad.net/bugs/1863021 and
-# https://github.com/eventlet/eventlet/issues/592
-import __original_module_threading as orig_threading
-import threading  # noqa
-orig_threading.current_thread.__globals__['_active'] = threading._active
 
 
 import os
@@ -152,10 +146,10 @@ LAUNCH_OPTIONS = {
 MISTRAL_TITLE = r"""
 |\\\    //|           ||                       ||
 ||\\\  //||      __   ||      __      __       ||
-|| \\\// || ||  //  ||||||  ||  \\\  //  \\\     ||
-||  \\/  ||     \\\    ||    ||     ||    \\\    ||
-||      || ||   \\\   ||    ||     ||    /\\\   ||
-||      || || __//   ||_// ||      \\\__// \\\_ ||
+|| \\\// || ||  //  ||||||  ||  \\  //  \\     ||
+||  \\/  ||     \\    ||    ||     ||    \\    ||
+||       || ||   \\   ||    ||     ||    /\\   ||
+||       || || __//   ||_// ||      \\__// \\_ ||
 
 Mistral Workflow Service, version %s
 """ % version.version_string
@@ -197,23 +191,6 @@ def get_properly_ordered_parameters():
     return args
 
 
-def override_keystone_options():
-    # TODO(wxy): This function is used for keeping backward compatibility.
-    # Remove it in Stein.
-    auth_opts = CONF['keystone_authtoken']
-
-    for opt, value in auth_opts.items():
-        if opt in CONF['keystone']:
-            default_value = auth_opts._group._opts[opt]['opt'].default
-
-            if default_value != value != CONF['keystone'][opt]:
-                LOG.warning("The config option '%s' in section "
-                            "[keystone_authtoken] has the same copy in "
-                            "[keystone]. Please add the same option to the "
-                            "[keystone] section to keep using it.", opt)
-                CONF.set_override(opt, value, group='keystone')
-
-
 def main():
     try:
         CONF.register_cli_opts(config.CLI_OPTS)
@@ -224,8 +201,6 @@ def main():
 
         logging.setup(CONF, 'Mistral')
 
-        override_keystone_options()
-
         # Please refer to the oslo.messaging documentation for transport
         # configuration. The default transport for oslo.messaging is
         # rabbitMQ. The available transport drivers are listed in the
