@@ -17,8 +17,8 @@
 import base64
 from email.header import decode_header
 from email import parser
-import mock
-import six
+from unittest import mock
+
 import testtools
 
 from mistral.actions import std_actions as std
@@ -54,6 +54,7 @@ class SendEmailActionTest(base.BaseTest):
         super(SendEmailActionTest, self).setUp()
         self.to_addrs = ["dz@example.com", "deg@example.com",
                          "xyz@example.com"]
+        self.reply_to = ['reply-to@example.com']
         self.cc_addrs = ['copy@example.com']
         self.bcc_addrs = ['hidden_copy@example.com']
         self.subject = "Multi word subject с русскими буквами"
@@ -64,6 +65,7 @@ class SendEmailActionTest(base.BaseTest):
         self.from_addr = "bot@example.com"
 
         self.to_addrs_str = ", ".join(self.to_addrs)
+        self.reply_to_str = ", ".join(self.reply_to)
         self.ctx = mock.Mock()
 
     @testtools.skipIf(not LOCAL_SMTPD, "Setup local smtpd to run it")
@@ -84,7 +86,6 @@ class SendEmailActionTest(base.BaseTest):
         self.smtp_server = 'mail.example.com:25'
         self.from_addr = "bot@example.com"
         self.smtp_password = 'secret'
-
         action = std.SendEmailAction(
             from_addr=self.from_addr,
             to_addrs=self.to_addrs,
@@ -151,26 +152,15 @@ class SendEmailActionTest(base.BaseTest):
 
         self.assertEqual(self.from_addr, message['from'])
         self.assertEqual(self.to_addrs_str, message['to'])
-        if six.PY3:
-            self.assertEqual(
-                self.subject,
-                decode_header(message['subject'])[0][0].decode('utf-8')
-            )
-        else:
-            self.assertEqual(
-                self.subject.decode('utf-8'),
-                decode_header(message['subject'])[0][0].decode('utf-8')
-            )
-        if six.PY3:
-            self.assertEqual(
-                self.body,
-                base64.b64decode(message.get_payload()).decode('utf-8')
-            )
-        else:
-            self.assertEqual(
-                self.body.decode('utf-8'),
-                base64.b64decode(message.get_payload()).decode('utf-8')
-            )
+        self.assertEqual(
+            self.subject,
+            decode_header(message['subject'])[0][0].decode('utf-8')
+        )
+
+        self.assertEqual(
+            self.body,
+            base64.b64decode(message.get_payload()).decode('utf-8')
+        )
 
     @mock.patch('smtplib.SMTP')
     def test_send_email_with_cc(self, smtp):
@@ -235,6 +225,35 @@ class SendEmailActionTest(base.BaseTest):
         self.assertEqual(self.from_addr, message['from'])
         self.assertEqual(self.to_addrs_str, message['to'])
 
+    @mock.patch('smtplib.SMTP')
+    def test_send_email_with_reply_to(self, smtp):
+        action = std.SendEmailAction(
+            from_addr=self.from_addr,
+            to_addrs=self.to_addrs,
+            reply_to=self.reply_to,
+            bcc_addrs=self.bcc_addrs,
+            smtp_server=self.smtp_server,
+            smtp_password=None,
+            subject=self.subject,
+            body=self.body
+        )
+
+        action.run(self.ctx)
+
+        smtp.assert_called_once_with(self.smtp_server)
+
+        sendmail = smtp.return_value.sendmail
+
+        self.assertTrue(sendmail.called, "should call sendmail")
+        self.assertEqual(
+            self.from_addr, sendmail.call_args[1]['from_addr'])
+
+        message = parser.Parser().parsestr(sendmail.call_args[1]['msg'])
+
+        self.assertEqual(self.from_addr, message['from'])
+        self.assertEqual(self.to_addrs_str, message['to'])
+        self.assertEqual(self.reply_to_str, message['reply-to'])
+
     @mock.patch('smtplib.SMTP')
     def test_send_email_html(self, smtp):
         action = std.SendEmailAction(
@@ -263,38 +282,20 @@ class SendEmailActionTest(base.BaseTest):
 
         self.assertEqual(self.from_addr, message['from'])
         self.assertEqual(self.to_addrs_str, message['to'])
-        if six.PY3:
-            self.assertEqual(
-                self.subject,
-                decode_header(message['subject'])[0][0].decode('utf-8')
-            )
-        else:
-            self.assertEqual(
-                self.subject.decode('utf-8'),
-                decode_header(message['subject'])[0][0].decode('utf-8')
-            )
+        self.assertEqual(
+            self.subject,
+            decode_header(message['subject'])[0][0].decode('utf-8')
+        )
         body_payload = message.get_payload(0).get_payload()
-        if six.PY3:
-            self.assertEqual(
-                self.body,
-                base64.b64decode(body_payload).decode('utf-8')
-            )
-        else:
-            self.assertEqual(
-                self.body.decode('utf-8'),
-                base64.b64decode(body_payload).decode('utf-8')
-            )
+        self.assertEqual(
+            self.body,
+            base64.b64decode(body_payload).decode('utf-8')
+        )
         html_body_payload = message.get_payload(1).get_payload()
-        if six.PY3:
-            self.assertEqual(
-                self.html_body,
-                base64.b64decode(html_body_payload).decode('utf-8')
-            )
-        else:
-            self.assertEqual(
-                self.html_body.decode('utf-8'),
-                base64.b64decode(html_body_payload).decode('utf-8')
-            )
+        self.assertEqual(
+            self.html_body,
+            base64.b64decode(html_body_payload).decode('utf-8')
+        )
 
     @mock.patch('smtplib.SMTP')
     def test_with_password(self, smtp):
