@@ -14,8 +14,8 @@
 #    limitations under the License.
 
 import copy
+from unittest import mock
 
-import mock
 import sqlalchemy as sa
 
 from mistral.db.v2 import api as db_api
@@ -169,6 +169,18 @@ class TestWorkbooksController(base.APITest):
         self.assertEqual(200, resp.status_int)
         self.assertDictEqual(WORKBOOK, resp.json)
 
+    @mock.patch('mistral.db.v2.api.get_workbook')
+    def test_get_with_fields_filter(self, mocked_get):
+        mocked_get.return_value = (WORKBOOK['id'], WORKBOOK['name'],)
+        resp = self.app.get('/v2/workbooks/123?fields=name')
+        expected = {
+            'id': WORKBOOK['id'],
+            'name': WORKBOOK['name'],
+        }
+
+        self.assertEqual(200, resp.status_int)
+        self.assertDictEqual(expected, resp.json)
+
     @mock.patch.object(db_api, "get_workbook", MOCK_WB_WITH_NAMESPACE)
     def test_get_with_namespace(self):
         resp = self.app.get('/v2/workbooks/123?namespace=xyz')
@@ -201,7 +213,7 @@ class TestWorkbooksController(base.APITest):
         resp = self.app.get('/v2/workbooks/123')
 
         self.assertEqual(200, resp.status_int)
-        self.assertTrue('project_id' in resp.json)
+        self.assertIn('project_id', resp.json)
 
     @mock.patch.object(workbooks, "update_workbook_v2", MOCK_UPDATED_WORKBOOK)
     def test_put(self):
@@ -236,6 +248,19 @@ class TestWorkbooksController(base.APITest):
         self.assertEqual(400, resp.status_int)
         self.assertIn("Invalid DSL", resp.body.decode())
 
+    @mock.patch.object(workbooks, "update_workbook_v2", MOCK_UPDATED_WORKBOOK)
+    def test_put_invalid_skip_validation(self):
+        self.override_config('validation_mode', 'enabled', 'api')
+
+        resp = self.app.put(
+            '/v2/workbooks?skip_validation',
+            WB_DEF_INVALID_MODEL_EXCEPTION,
+            headers={'Content-Type': 'text/plain'},
+            expect_errors=True
+        )
+
+        self.assertEqual(200, resp.status_int)
+
     @mock.patch.object(db_api, "update_workbook")
     @mock.patch.object(db_api, "create_or_update_workflow_definition")
     @mock.patch.object(db_api, "create_or_update_action_definition")
@@ -314,6 +339,18 @@ class TestWorkbooksController(base.APITest):
         self.assertEqual(400, resp.status_int)
         self.assertIn("Invalid DSL", resp.body.decode())
 
+    def test_post_invalid_skip_validation(self):
+        self.override_config('validation_mode', 'enabled', 'api')
+
+        resp = self.app.post(
+            '/v2/workbooks?skip_validation',
+            WB_DEF_INVALID_MODEL_EXCEPTION,
+            headers={'Content-Type': 'text/plain'},
+            expect_errors=True
+        )
+
+        self.assertEqual(201, resp.status_int)
+
     @mock.patch.object(db_api, "create_workbook")
     @mock.patch.object(db_api, "create_or_update_workflow_definition")
     @mock.patch.object(db_api, "create_or_update_action_definition")
