@@ -12,7 +12,7 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import mock
+from unittest import mock
 
 from oslo_config import cfg
 import testtools
@@ -161,6 +161,10 @@ workflows:
     type: direct
 
     tasks:
+      t0:
+        action: std.noop
+        on-success:
+          - t3
       t1:
         action: std.echo output="Task 1"
         on-success:
@@ -335,10 +339,10 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
             task_execs = wf_ex.task_executions
 
-        self.assertEqual(states.ERROR, wf_ex.state)
-        self.assertIsNotNone(wf_ex.state_info)
-        self.assertEqual(3, len(task_execs))
-        self.assertDictEqual(env, wf_ex.params['env'])
+            self.assertEqual(states.ERROR, wf_ex.state)
+            self.assertIsNotNone(wf_ex.state_info)
+            self.assertEqual(3, len(task_execs))
+            self.assertDictEqual(env, wf_ex.params['env'])
 
         task_10_ex = self._assert_single_item(task_execs, name='t10')
         task_21_ex = self._assert_single_item(task_execs, name='t21')
@@ -457,56 +461,6 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
             task_30_action_exs[0].input
         )
 
-    @mock.patch.object(
-        std_actions.EchoAction,
-        'run',
-        mock.MagicMock(
-            side_effect=[
-                'Task 1',               # Mock task1 success for initial run.
-                exc.ActionException()   # Mock task2 exception for initial run.
-            ]
-        )
-    )
-    def test_rerun_from_prev_step(self):
-        wb_service.create_workbook_v2(SIMPLE_WORKBOOK)
-
-        # Run workflow and fail task.
-        wf_ex = self.engine.start_workflow('wb1.wf1')
-
-        self.await_workflow_error(wf_ex.id)
-
-        with db_api.transaction():
-            wf_ex = db_api.get_workflow_execution(wf_ex.id)
-
-            task_execs = wf_ex.task_executions
-
-        self.assertEqual(states.ERROR, wf_ex.state)
-        self.assertIsNotNone(wf_ex.state_info)
-
-        self.assertEqual(2, len(task_execs))
-
-        task_1_ex = self._assert_single_item(
-            task_execs,
-            name='t1',
-            state=states.SUCCESS
-        )
-        task_2_ex = self._assert_single_item(
-            task_execs,
-            name='t2',
-            state=states.ERROR
-        )
-
-        self.assertIsNotNone(task_2_ex.state_info)
-
-        # Resume workflow and re-run failed task.
-        e = self.assertRaises(
-            exc.MistralError,
-            self.engine.rerun_workflow,
-            task_1_ex.id
-        )
-
-        self.assertIn('not supported', str(e))
-
     @mock.patch.object(
         std_actions.EchoAction,
         'run',
@@ -818,12 +772,14 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         self.assertEqual(states.ERROR, wf_ex.state)
         self.assertIsNotNone(wf_ex.state_info)
-        self.assertEqual(3, len(task_execs))
+        self.assertEqual(4, len(task_execs))
 
+        task_0_ex = self._assert_single_item(task_execs, name='t0')
         task_1_ex = self._assert_single_item(task_execs, name='t1')
         task_2_ex = self._assert_single_item(task_execs, name='t2')
         task_3_ex = self._assert_single_item(task_execs, name='t3')
 
+        self.assertEqual(states.SUCCESS, task_0_ex.state)
         self.assertEqual(states.SUCCESS, task_1_ex.state)
         self.assertEqual(states.SUCCESS, task_2_ex.state)
         self.assertEqual(states.ERROR, task_3_ex.state)
@@ -847,12 +803,23 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
-        self.assertEqual(3, len(task_execs))
+        self.assertEqual(4, len(task_execs))
 
+        task_0_ex = self._assert_single_item(task_execs, name='t0')
         task_1_ex = self._assert_single_item(task_execs, name='t1')
         task_2_ex = self._assert_single_item(task_execs, name='t2')
         task_3_ex = self._assert_single_item(task_execs, name='t3')
 
+        # Check action executions of task 0.
+        self.assertEqual(states.SUCCESS, task_0_ex.state)
+
+        task_0_action_exs = db_api.get_action_executions(
+            task_execution_id=task_0_ex.id
+        )
+
+        self.assertEqual(1, len(task_0_action_exs))
+        self.assertEqual(states.SUCCESS, task_0_action_exs[0].state)
+
         # Check action executions of task 1.
         self.assertEqual(states.SUCCESS, task_1_ex.state)
 
@@ -906,79 +873,29 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         # Run workflow and fail task.
         wf_ex = self.engine.start_workflow('wb1.wf1')
 
-        with db_api.transaction():
-            wf_ex = db_api.get_workflow_execution(wf_ex.id)
-            task_execs = wf_ex.task_executions
-
-        task_1_ex = self._assert_single_item(task_execs, name='t1')
-        task_2_ex = self._assert_single_item(task_execs, name='t2')
-
-        self.await_task_error(task_1_ex.id)
-        self.await_task_error(task_2_ex.id)
-
         self.await_workflow_error(wf_ex.id)
 
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
             task_execs = wf_ex.task_executions
 
-        self.assertEqual(states.ERROR, wf_ex.state)
-        self.assertIsNotNone(wf_ex.state_info)
-        self.assertEqual(2, len(task_execs))
+        # NOTE(arnaud): we used to tests the tasks results here but we can't
+        # be sure about which one is failing, which one has succeeded or is
+        # still in waiting state.
+        # What we know is that, when a task fails, and if there is no condition
+        # to continue the worflow (like on-complete), then the workflow will
+        # fail and all other tasks may be blocked in WAITING state.
 
-        task_1_ex = self._assert_single_item(task_execs, name='t1')
+        # Resume workflow and re-run failed tasks
+        for task_exec in task_execs:
+            if task_exec.state == states.ERROR:
+                wf_ex = self.engine.rerun_workflow(task_exec.id)
 
-        self.assertEqual(states.ERROR, task_1_ex.state)
-        self.assertIsNotNone(task_1_ex.state_info)
-
-        task_2_ex = self._assert_single_item(task_execs, name='t2')
-
-        self.assertEqual(states.ERROR, task_2_ex.state)
-        self.assertIsNotNone(task_2_ex.state_info)
-
-        # Resume workflow and re-run failed task.
-        wf_ex = self.engine.rerun_workflow(task_1_ex.id)
-
-        self.assertEqual(states.RUNNING, wf_ex.state)
+        # This is prone to race conditions, but let's see if that works
+        self.await_workflow_running(wf_ex.id)
         self.assertIsNone(wf_ex.state_info)
 
-        with db_api.transaction():
-            wf_ex = db_api.get_workflow_execution(wf_ex.id)
-            task_execs = wf_ex.task_executions
-
-        # Wait for the task to succeed.
-        task_1_ex = self._assert_single_item(task_execs, name='t1')
-
-        self.await_task_success(task_1_ex.id)
-
-        self.await_workflow_error(wf_ex.id)
-
-        with db_api.transaction():
-            wf_ex = db_api.get_workflow_execution(wf_ex.id)
-            task_execs = wf_ex.task_executions
-
-        self.assertEqual(states.ERROR, wf_ex.state)
-        self.assertIsNotNone(wf_ex.state_info)
-        self.assertEqual(3, len(task_execs))
-
-        task_1_ex = self._assert_single_item(task_execs, name='t1')
-        task_2_ex = self._assert_single_item(task_execs, name='t2')
-        task_3_ex = self._assert_single_item(task_execs, name='t3')
-
-        self.assertEqual(states.SUCCESS, task_1_ex.state)
-        self.assertEqual(states.ERROR, task_2_ex.state)
-        self.assertEqual(states.ERROR, task_3_ex.state)
-
-        # Resume workflow and re-run failed task.
-        wf_ex = self.engine.rerun_workflow(task_2_ex.id)
-
-        self.assertEqual(states.RUNNING, wf_ex.state)
-        self.assertIsNone(wf_ex.state_info)
-
-        # Join now should finally complete.
-        self.await_task_success(task_3_ex.id)
-
-        # Wait for the workflow to succeed.
+        # Now wait for the workflow to finish
         self.await_workflow_success(wf_ex.id)
 
         with db_api.transaction():
@@ -987,12 +904,22 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
-        self.assertEqual(3, len(task_execs))
+        self.assertEqual(4, len(task_execs))
 
+        task_0_ex = self._assert_single_item(task_execs, name='t0')
         task_1_ex = self._assert_single_item(task_execs, name='t1')
         task_2_ex = self._assert_single_item(task_execs, name='t2')
         task_3_ex = self._assert_single_item(task_execs, name='t3')
 
+        # Check action executions of task 0.
+        self.assertEqual(states.SUCCESS, task_0_ex.state)
+        self.assertIsNone(task_0_ex.state_info)
+
+        task_0_action_exs = db_api.get_action_executions(
+            task_execution_id=task_0_ex.id
+        )
+
+        self.assertEqual(1, len(task_0_action_exs))
         # Check action executions of task 1.
         self.assertEqual(states.SUCCESS, task_1_ex.state)
         self.assertIsNone(task_1_ex.state_info)
@@ -1021,7 +948,7 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self._assert_single_item(task_2_action_exs, state=states.SUCCESS)
         self._assert_single_item(task_2_action_exs, state=states.ERROR)
 
-        # Check action executions of task 3.
+        # Check there is exactly 1 action execution of task 3.
         self.assertEqual(states.SUCCESS, task_3_ex.state)
 
         task_3_action_exs = db_api.get_action_executions(
@@ -1480,3 +1407,74 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertEqual(6, len(action_executions))
         self.assertTrue(all(a.state == states.ERROR
                             for a in action_executions))
+
+    @mock.patch.object(
+        std_actions.NoOpAction,
+        'run',
+        mock.MagicMock(
+            side_effect=[
+                exc.ActionException(),
+                'Success'
+            ]
+        )
+    )
+    def test_rerun_sub_workflow(self):
+        wf_service.create_workflows("""---
+        version: '2.0'
+        wf1:
+          tasks:
+            task1:
+              workflow: wf2
+        wf2:
+          tasks:
+            task2:
+              workflow: wf3
+        wf3:
+          tasks:
+            task3:
+              action: std.noop""")
+
+        # Run workflow and fail task.
+        wf1_ex = self.engine.start_workflow('wf1')
+
+        self.await_workflow_error(wf1_ex.id)
+
+        with db_api.transaction():
+            wf_exs = db_api.get_workflow_executions()
+            task_exs = db_api.get_task_executions()
+
+            self.assertEqual(3, len(wf_exs),
+                             'The number of workflow executions')
+            self.assertEqual(3, len(task_exs),
+                             'The number of task executions')
+
+            for wf_ex in wf_exs:
+                self.assertEqual(states.ERROR, wf_ex.state,
+                                 'The executions must fail the first time')
+            for task_ex in task_exs:
+                self.assertEqual(states.ERROR, task_ex.state,
+                                 'The tasks must fail the first time')
+
+            wf3_ex = self._assert_single_item(wf_exs, name='wf3')
+            task3_ex = self._assert_single_item(wf3_ex.task_executions,
+                                                name="task3")
+
+        self.engine.rerun_workflow(task3_ex.id)
+
+        self.await_workflow_success(wf1_ex.id)
+
+        with db_api.transaction():
+            wf_exs = db_api.get_workflow_executions()
+            task_exs = db_api.get_task_executions()
+
+            self.assertEqual(3, len(wf_exs),
+                             'The number of workflow executions')
+            self.assertEqual(3, len(task_exs),
+                             'The number of task executions')
+
+            for wf_ex in wf_exs:
+                self.assertEqual(states.SUCCESS, wf_ex.state,
+                                 'The executions must success the second time')
+            for task_ex in task_exs:
+                self.assertEqual(states.SUCCESS, task_ex.state,
+                                 'The tasks must success the second time')
