@@ -22,6 +22,143 @@ from mistral.tests.unit.lang.v2 import base
 from mistral_lib import utils
 
 
+def generate_test_function(default, expect_error):
+    """Function used to dynamically generate tests"""
+    def test(self):
+        overlay = {'test': {'task-defaults': {}}}
+        utils.merge_dicts(overlay['test']['task-defaults'], default)
+        self._parse_dsl_spec(
+            add_tasks=True,
+            changes=overlay,
+            expect_error=expect_error
+        )
+    return test
+
+
+class WorkflowSpecValidationTestDefault(base.WorkflowSpecValidationTestCase):
+    pass
+
+
+tests = [
+    ({'on-success': ['email']}, False),
+    ({'on-success': [{'email': '<% 1 %>'}]}, False),
+    ({'on-success': [{'email': '<% 1 %>'}, 'echo']}, False),
+    ({'on-success': [{'email': '<% $.v1 in $.v2 %>'}]}, False),
+    ({'on-success': [{'email': '<% * %>'}]}, True),
+    ({'on-success': [{'email': '{{ 1 }}'}]}, False),
+    ({'on-success': [{'email': '{{ 1 }}'}, 'echo']}, False),
+    ({'on-success': [{'email': '{{ _.v1 in _.v2 }}'}]}, False),
+    ({'on-success': [{'email': '{{ * }}'}]}, True),
+    ({'on-success': 'email'}, False),
+    ({'on-success': None}, True),
+    ({'on-success': ['']}, True),
+    ({'on-success': []}, True),
+    ({'on-success': ['email', 'email']}, True),
+    ({'on-success': ['email', 12345]}, True),
+    ({'on-error': ['email']}, False),
+    ({'on-error': [{'email': '<% 1 %>'}]}, False),
+    ({'on-error': [{'email': '<% 1 %>'}, 'echo']}, False),
+    ({'on-error': [{'email': '<% $.v1 in $.v2 %>'}]}, False),
+    ({'on-error': [{'email': '<% * %>'}]}, True),
+    ({'on-error': [{'email': '{{ 1 }}'}]}, False),
+    ({'on-error': [{'email': '{{ 1 }}'}, 'echo']}, False),
+    ({'on-error': [{'email': '{{ _.v1 in _.v2 }}'}]}, False),
+    ({'on-error': [{'email': '{{ * }}'}]}, True),
+    ({'on-error': 'email'}, False),
+    ({'on-error': None}, True),
+    ({'on-error': ['']}, True),
+    ({'on-error': []}, True),
+    ({'on-error': ['email', 'email']}, True),
+    ({'on-error': ['email', 12345]}, True),
+    ({'on-complete': ['email']}, False),
+    ({'on-complete': [{'email': '<% 1 %>'}]}, False),
+    ({'on-complete': [{'email': '<% 1 %>'}, 'echo']}, False),
+    ({'on-complete': [{'email': '<% $.v1 in $.v2 %>'}]}, False),
+    ({'on-complete': [{'email': '<% * %>'}]}, True),
+    ({'on-complete': [{'email': '{{ 1 }}'}]}, False),
+    ({'on-complete': [{'email': '{{ 1 }}'}, 'echo']}, False),
+    ({'on-complete': [{'email': '{{ _.v1 in _.v2 }}'}]}, False),
+    ({'on-complete': [{'email': '{{ * }}'}]}, True),
+    ({'on-complete': 'email'}, False),
+    ({'on-complete': None}, True),
+    ({'on-complete': ['']}, True),
+    ({'on-complete': []}, True),
+    ({'on-complete': ['email', 'email']}, True),
+    ({'on-complete': ['email', 12345]}, True),
+    ({'requires': ''}, True),
+    ({'requires': []}, True),
+    ({'requires': ['']}, True),
+    ({'requires': None}, True),
+    ({'requires': 12345}, True),
+    ({'requires': ['echo']}, False),
+    ({'requires': ['echo', 'get']}, False),
+    ({'requires': 'echo'}, False),
+    ({'retry': {'count': 3, 'delay': 1}}, False),
+    ({'retry': {'count': '<% 3 %>', 'delay': 1}}, False),
+    ({'retry': {'count': '<% * %>', 'delay': 1}}, True),
+    ({'retry': {'count': 3, 'delay': '<% 1 %>'}}, False),
+    ({'retry': {'count': 3, 'delay': '<% * %>'}}, True),
+    ({'retry': {'count': '{{ 3 }}', 'delay': 1}}, False),
+    ({'retry': {'count': '{{ * }}', 'delay': 1}}, True),
+    ({'retry': {'count': 3, 'delay': '{{ 1 }}'}}, False),
+    ({'retry': {'count': 3, 'delay': '{{ * }}'}}, True),
+    ({'retry': {'count': -3, 'delay': 1}}, True),
+    ({'retry': {'count': 3, 'delay': -1}}, True),
+    ({'retry': {'count': '3', 'delay': 1}}, True),
+    ({'retry': {'count': 3, 'delay': '1'}}, True),
+    ({'retry': None}, True),
+    ({'wait-before': 1}, False),
+    ({'wait-before': '<% 1 %>'}, False),
+    ({'wait-before': '<% * %>'}, True),
+    ({'wait-before': '{{ 1 }}'}, False),
+    ({'wait-before': '{{ * }}'}, True),
+    ({'wait-before': -1}, True),
+    ({'wait-before': 1.1}, True),
+    ({'wait-before': '1'}, True),
+    ({'wait-after': 1}, False),
+    ({'wait-after': '<% 1 %>'}, False),
+    ({'wait-after': '<% * %>'}, True),
+    ({'wait-after': '{{ 1 }}'}, False),
+    ({'wait-after': '{{ * }}'}, True),
+    ({'wait-after': -1}, True),
+    ({'wait-after': 1.1}, True),
+    ({'wait-after': '1'}, True),
+    ({'timeout': 300}, False),
+    ({'timeout': '<% 300 %>'}, False),
+    ({'timeout': '<% * %>'}, True),
+    ({'timeout': '{{ 300 }}'}, False),
+    ({'timeout': '{{ * }}'}, True),
+    ({'timeout': -300}, True),
+    ({'timeout': 300.1}, True),
+    ({'timeout': '300'}, True),
+    ({'pause-before': False}, False),
+    ({'pause-before': '<% False %>'}, False),
+    ({'pause-before': '<% * %>'}, True),
+    ({'pause-before': '{{ False }}'}, False),
+    ({'pause-before': '{{ * }}'}, True),
+    ({'pause-before': 'False'}, True),
+    ({'concurrency': 10}, False),
+    ({'concurrency': '<% 10 %>'}, False),
+    ({'concurrency': '<% * %>'}, True),
+    ({'concurrency': '{{ 10 }}'}, False),
+    ({'concurrency': '{{ * }}'}, True),
+    ({'concurrency': -10}, True),
+    ({'concurrency': 10.1}, True),
+    ({'concurrency': '10'}, True),
+    ({'safe-rerun': True}, False),
+    ({'safe-rerun': False}, False),
+    ({'safe-rerun': '<% false %>'}, False),
+    ({'safe-rerun': '<% true %>'}, False),
+    ({'safe-rerun': '<% * %>'}, True),
+    ({'safe-rerun': None}, True)
+]
+
+for index, test in enumerate(tests):
+    test_name = f"test_index_{index}"
+    test_func = generate_test_function(test[0], test[1])
+    setattr(WorkflowSpecValidationTestDefault, test_name, test_func)
+
+
 class WorkflowSpecValidationTest(base.WorkflowSpecValidationTestCase):
     def test_workflow_types(self):
         tests = [
@@ -277,132 +414,6 @@ class WorkflowSpecValidationTest(base.WorkflowSpecValidationTestCase):
                 expect_error=expect_error
             )
 
-    def test_task_defaults(self):
-        tests = [
-            ({'on-success': ['email']}, False),
-            ({'on-success': [{'email': '<% 1 %>'}]}, False),
-            ({'on-success': [{'email': '<% 1 %>'}, 'echo']}, False),
-            ({'on-success': [{'email': '<% $.v1 in $.v2 %>'}]}, False),
-            ({'on-success': [{'email': '<% * %>'}]}, True),
-            ({'on-success': [{'email': '{{ 1 }}'}]}, False),
-            ({'on-success': [{'email': '{{ 1 }}'}, 'echo']}, False),
-            ({'on-success': [{'email': '{{ _.v1 in _.v2 }}'}]}, False),
-            ({'on-success': [{'email': '{{ * }}'}]}, True),
-            ({'on-success': 'email'}, False),
-            ({'on-success': None}, True),
-            ({'on-success': ['']}, True),
-            ({'on-success': []}, True),
-            ({'on-success': ['email', 'email']}, True),
-            ({'on-success': ['email', 12345]}, True),
-            ({'on-error': ['email']}, False),
-            ({'on-error': [{'email': '<% 1 %>'}]}, False),
-            ({'on-error': [{'email': '<% 1 %>'}, 'echo']}, False),
-            ({'on-error': [{'email': '<% $.v1 in $.v2 %>'}]}, False),
-            ({'on-error': [{'email': '<% * %>'}]}, True),
-            ({'on-error': [{'email': '{{ 1 }}'}]}, False),
-            ({'on-error': [{'email': '{{ 1 }}'}, 'echo']}, False),
-            ({'on-error': [{'email': '{{ _.v1 in _.v2 }}'}]}, False),
-            ({'on-error': [{'email': '{{ * }}'}]}, True),
-            ({'on-error': 'email'}, False),
-            ({'on-error': None}, True),
-            ({'on-error': ['']}, True),
-            ({'on-error': []}, True),
-            ({'on-error': ['email', 'email']}, True),
-            ({'on-error': ['email', 12345]}, True),
-            ({'on-complete': ['email']}, False),
-            ({'on-complete': [{'email': '<% 1 %>'}]}, False),
-            ({'on-complete': [{'email': '<% 1 %>'}, 'echo']}, False),
-            ({'on-complete': [{'email': '<% $.v1 in $.v2 %>'}]}, False),
-            ({'on-complete': [{'email': '<% * %>'}]}, True),
-            ({'on-complete': [{'email': '{{ 1 }}'}]}, False),
-            ({'on-complete': [{'email': '{{ 1 }}'}, 'echo']}, False),
-            ({'on-complete': [{'email': '{{ _.v1 in _.v2 }}'}]}, False),
-            ({'on-complete': [{'email': '{{ * }}'}]}, True),
-            ({'on-complete': 'email'}, False),
-            ({'on-complete': None}, True),
-            ({'on-complete': ['']}, True),
-            ({'on-complete': []}, True),
-            ({'on-complete': ['email', 'email']}, True),
-            ({'on-complete': ['email', 12345]}, True),
-            ({'requires': ''}, True),
-            ({'requires': []}, True),
-            ({'requires': ['']}, True),
-            ({'requires': None}, True),
-            ({'requires': 12345}, True),
-            ({'requires': ['echo']}, False),
-            ({'requires': ['echo', 'get']}, False),
-            ({'requires': 'echo'}, False),
-            ({'retry': {'count': 3, 'delay': 1}}, False),
-            ({'retry': {'count': '<% 3 %>', 'delay': 1}}, False),
-            ({'retry': {'count': '<% * %>', 'delay': 1}}, True),
-            ({'retry': {'count': 3, 'delay': '<% 1 %>'}}, False),
-            ({'retry': {'count': 3, 'delay': '<% * %>'}}, True),
-            ({'retry': {'count': '{{ 3 }}', 'delay': 1}}, False),
-            ({'retry': {'count': '{{ * }}', 'delay': 1}}, True),
-            ({'retry': {'count': 3, 'delay': '{{ 1 }}'}}, False),
-            ({'retry': {'count': 3, 'delay': '{{ * }}'}}, True),
-            ({'retry': {'count': -3, 'delay': 1}}, True),
-            ({'retry': {'count': 3, 'delay': -1}}, True),
-            ({'retry': {'count': '3', 'delay': 1}}, True),
-            ({'retry': {'count': 3, 'delay': '1'}}, True),
-            ({'retry': None}, True),
-            ({'wait-before': 1}, False),
-            ({'wait-before': '<% 1 %>'}, False),
-            ({'wait-before': '<% * %>'}, True),
-            ({'wait-before': '{{ 1 }}'}, False),
-            ({'wait-before': '{{ * }}'}, True),
-            ({'wait-before': -1}, True),
-            ({'wait-before': 1.1}, True),
-            ({'wait-before': '1'}, True),
-            ({'wait-after': 1}, False),
-            ({'wait-after': '<% 1 %>'}, False),
-            ({'wait-after': '<% * %>'}, True),
-            ({'wait-after': '{{ 1 }}'}, False),
-            ({'wait-after': '{{ * }}'}, True),
-            ({'wait-after': -1}, True),
-            ({'wait-after': 1.1}, True),
-            ({'wait-after': '1'}, True),
-            ({'timeout': 300}, False),
-            ({'timeout': '<% 300 %>'}, False),
-            ({'timeout': '<% * %>'}, True),
-            ({'timeout': '{{ 300 }}'}, False),
-            ({'timeout': '{{ * }}'}, True),
-            ({'timeout': -300}, True),
-            ({'timeout': 300.1}, True),
-            ({'timeout': '300'}, True),
-            ({'pause-before': False}, False),
-            ({'pause-before': '<% False %>'}, False),
-            ({'pause-before': '<% * %>'}, True),
-            ({'pause-before': '{{ False }}'}, False),
-            ({'pause-before': '{{ * }}'}, True),
-            ({'pause-before': 'False'}, True),
-            ({'concurrency': 10}, False),
-            ({'concurrency': '<% 10 %>'}, False),
-            ({'concurrency': '<% * %>'}, True),
-            ({'concurrency': '{{ 10 }}'}, False),
-            ({'concurrency': '{{ * }}'}, True),
-            ({'concurrency': -10}, True),
-            ({'concurrency': 10.1}, True),
-            ({'concurrency': '10'}, True),
-            ({'safe-rerun': True}, False),
-            ({'safe-rerun': False}, False),
-            ({'safe-rerun': '<% false %>'}, False),
-            ({'safe-rerun': '<% true %>'}, False),
-            ({'safe-rerun': '<% * %>'}, True),
-            ({'safe-rerun': None}, True)
-        ]
-
-        for default, expect_error in tests:
-            overlay = {'test': {'task-defaults': {}}}
-
-            utils.merge_dicts(overlay['test']['task-defaults'], default)
-
-            self._parse_dsl_spec(
-                add_tasks=True,
-                changes=overlay,
-                expect_error=expect_error
-            )
-
     def test_invalid_item(self):
         overlay = {'name': 'invalid'}
 
