@@ -13,18 +13,20 @@
 #    limitations under the License.
 
 import datetime
-import eventlet
-import mock
+import time
+from unittest import mock
+
 from oslo_config import cfg
+from oslo_utils import timeutils
 
-from mistral.engine.rpc_backend import rpc
 from mistral import exceptions as exc
+from mistral.rpc import clients as rpc
 from mistral.services import periodic
 from mistral.services import security
 from mistral.services import triggers as t_s
 from mistral.services import workflows
 from mistral.tests.unit import base
-from mistral import utils
+from mistral_lib import utils
 
 # Use the set_default method to set value otherwise in certain test cases
 # the change in value is not permanent.
@@ -54,9 +56,9 @@ def new_advance_cron_trigger(ct):
     another coroutine the process_cron_triggers_v2 will finish looping
     over all the cron triggers in one coroutine without any sharing at all.
     """
-    eventlet.sleep()
+    time.sleep()
     modified = advance_cron_trigger_orig(ct)
-    eventlet.sleep()
+    time.sleep()
 
     return modified
 
@@ -182,8 +184,8 @@ class TriggerServiceV2Test(base.DbTestCase):
             datetime.datetime(2010, 8, 25)
         )
 
-        self.assertIn('Invalid input', exception.message)
-        self.assertIn('some_wf', exception.message)
+        self.assertIn('Invalid input', str(exception))
+        self.assertIn('some_wf', str(exception))
 
     def test_oneshot_trigger_create(self):
         trigger = t_s.create_cron_trigger(
@@ -224,6 +226,39 @@ class TriggerServiceV2Test(base.DbTestCase):
 
         self.assertEqual('my_trust_id', trigger.trust_id)
 
+    @mock.patch.object(security, 'create_trust',
+                       type('trust', (object,), {'id': 'my_trust_id'}))
+    @mock.patch.object(security, 'create_context')
+    @mock.patch.object(rpc.EngineClient, 'start_workflow', mock.Mock())
+    @mock.patch(
+        'mistral.services.periodic.advance_cron_trigger',
+        mock.MagicMock(side_effect=new_advance_cron_trigger)
+    )
+    @mock.patch.object(security, 'delete_trust')
+    def test_create_delete_trust_in_trigger(self, delete_trust, create_ctx):
+        create_ctx.return_value = self.ctx
+        cfg.CONF.set_default('auth_enable', True, group='pecan')
+        trigger_thread = periodic.setup()
+        self.addCleanup(trigger_thread.stop)
+        self.addCleanup(
+            cfg.CONF.set_default, 'auth_enable',
+            False, group='pecan'
+        )
+
+        t_s.create_cron_trigger(
+            'trigger-%s' % utils.generate_unicode_uuid(),
+            self.wf.name,
+            {},
+            {},
+            '* * * * * *',
+            None,
+            1,
+            datetime.datetime(2010, 8, 25)
+        )
+
+        time.sleep(1)
+        self.assertEqual(0, delete_trust.call_count)
+
     def test_get_trigger_in_correct_orders(self):
         t1_name = 'trigger-%s' % utils.generate_unicode_uuid()
 
@@ -231,11 +266,8 @@ class TriggerServiceV2Test(base.DbTestCase):
             t1_name,
             self.wf.name,
             {},
-            {},
-            '*/5 * * * *',
-            None,
-            None,
-            datetime.datetime(2010, 8, 25)
+            pattern='*/5 * * * *',
+            start_time=datetime.datetime(2010, 8, 25)
         )
 
         t2_name = 'trigger-%s' % utils.generate_unicode_uuid()
@@ -244,11 +276,8 @@ class TriggerServiceV2Test(base.DbTestCase):
             t2_name,
             self.wf.name,
             {},
-            {},
-            '*/1 * * * *',
-            None,
-            None,
-            datetime.datetime(2010, 8, 22)
+            pattern='*/1 * * * *',
+            start_time=datetime.datetime(2010, 8, 22)
         )
 
         t3_name = 'trigger-%s' % utils.generate_unicode_uuid()
@@ -257,11 +286,8 @@ class TriggerServiceV2Test(base.DbTestCase):
             t3_name,
             self.wf.name,
             {},
-            {},
-            '*/2 * * * *',
-            None,
-            None,
-            datetime.datetime(2010, 9, 21)
+            pattern='*/2 * * * *',
+            start_time=datetime.datetime(2010, 9, 21)
         )
 
         t4_name = 'trigger-%s' % utils.generate_unicode_uuid()
@@ -270,11 +296,8 @@ class TriggerServiceV2Test(base.DbTestCase):
             t4_name,
             self.wf.name,
             {},
-            {},
-            '*/3 * * * *',
-            None,
-            None,
-            datetime.datetime.now() + datetime.timedelta(0, 50)
+            pattern='*/3 * * * *',
+            start_time=timeutils.utcnow() + datetime.timedelta(0, 50)
         )
 
         trigger_names = [t.name for t in t_s.get_next_cron_triggers()]
@@ -320,12 +343,18 @@ class TriggerServiceV2Test(base.DbTestCase):
 
         # Wait some more and make sure there are no more than 'trigger_count'
         # executions.
-        eventlet.sleep(5)
+        time.sleep(5)
 
         self.assertEqual(trigger_count, start_wf_mock.call_count)
 
     def _wait_for_single_execution_with_multiple_processes(self, trigger_count,
                                                            start_wf_mock):
-        eventlet.sleep(1)
+        time.sleep(1)
 
         return trigger_count == start_wf_mock.call_count
+
+    def test_get_next_execution_time(self):
+        pattern = '*/20 * * * *'
+        start_time = datetime.datetime(2016, 3, 22, 23, 40)
+        result = t_s.get_next_execution_time(pattern, start_time)
+        self.assertEqual(result, datetime.datetime(2016, 3, 23, 0, 0))
