@@ -57,12 +57,14 @@ class StatesModuleTest(base.BaseTest):
         self.assertFalse(s.is_valid_transition(s.SUCCESS, s.RUNNING))
         self.assertFalse(s.is_valid_transition(s.SUCCESS, s.ERROR))
         self.assertFalse(s.is_valid_transition(s.SUCCESS, s.PAUSED))
+        self.assertFalse(s.is_valid_transition(s.SUCCESS, s.SKIPPED))
         self.assertFalse(s.is_valid_transition(s.SUCCESS, s.RUNNING_DELAYED))
         self.assertFalse(s.is_valid_transition(s.SUCCESS, s.IDLE))
 
         # From ERROR
         self.assertTrue(s.is_valid_transition(s.ERROR, s.ERROR))
         self.assertTrue(s.is_valid_transition(s.ERROR, s.RUNNING))
+        self.assertTrue(s.is_valid_transition(s.ERROR, s.SKIPPED))
         self.assertFalse(s.is_valid_transition(s.ERROR, s.PAUSED))
         self.assertFalse(s.is_valid_transition(s.ERROR, s.RUNNING_DELAYED))
         self.assertFalse(s.is_valid_transition(s.ERROR, s.SUCCESS))
