@@ -30,10 +30,6 @@ from mistral import exceptions as exc
 from mistral import expressions as expr
 
 
-# Thread local storage.
-_th_loc_storage = threading.local()
-
-
 @contextlib.contextmanager
 def tempdir(**kwargs):
     argdict = kwargs.copy()
@@ -168,3 +164,18 @@ def evaluate_object_fields(obj, ctx):
 
     for k, v in evaluated_fields.items():
         setattr(obj, k, v)
+
+
+class ThreadWithException(threading.Thread):
+    """Thread capturing exception"""
+    def run(self):
+        self.exception = None
+        try:
+            super().run()
+        except Exception as e:
+            self.exception = e
+
+    def join(self, *args, **kwargs):
+        super().join(*args, **kwargs)
+        if self.exception:
+            raise self.exception
