@@ -1,6 +1,7 @@
 # Copyright 2014 - Mirantis, Inc.
 # Copyright 2016 - Brocade Communications Systems, Inc.
 # Copyright 2018 - Nokia, Inc.
+# Copyright 2022 - NetCracker Technology Corp.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -17,6 +18,7 @@
 import functools
 import json
 
+from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
 import pecan
@@ -121,6 +123,14 @@ def validate_fields(fields, object_fields):
         )
 
 
+def fields_list_to_cls_fields_tuple(model, f):
+    if not f:
+        return ()
+    return tuple(
+        [getattr(model, str(field)) for field in f]
+    )
+
+
 def filters_to_dict(**kwargs):
     """Return only non-null values
 
@@ -157,7 +167,7 @@ def get_all(list_cls, cls, get_all_function, get_function,
                       Default: ['asc'].
     :param fields: Optional. A specified list of fields of the resource to
                    be returned. 'id' will be included automatically in
-                   fields if it's provided, since it will be used when
+                   fields if it's not provided, since it will be used when
                    constructing 'next' link.
     :param filters: Optional. A specified dictionary of filters to match.
     :param all_projects: Optional. Get resources of all projects.
@@ -177,7 +187,7 @@ def get_all(list_cls, cls, get_all_function, get_function,
     insecure = False
 
     if (all_projects or
-            (auth_ctx.ctx().is_admin and filters.get('project_id', ''))):
+            (auth_ctx.ctx().is_admin)):
         insecure = True
 
     marker_obj = None
@@ -256,12 +266,15 @@ def get_all(list_cls, cls, get_all_function, get_function,
 
 
 class MistralRetrying(tenacity.Retrying):
-    def call(self, fn, *args, **kwargs):
+    def __call__(self, fn, *args, **kwargs):
         try:
-            return super(MistralRetrying, self).call(fn, *args, **kwargs)
+            return super(MistralRetrying, self).__call__(fn, *args, **kwargs)
         except tenacity.RetryError:
             raise exc.MistralError("The service is temporarily unavailable")
 
+    def call(self, fn, *args, **kwargs):
+        return self.__call__(fn, *args, **kwargs)
+
 
 def create_db_retry_object():
     return MistralRetrying(
@@ -291,3 +304,25 @@ def load_deferred_fields(ex, fields):
         hasattr(ex, f)
 
     return ex
+
+
+def clear_sensitive_headers(dirty_data):
+    if not dirty_data:
+        return dirty_data
+    clean_data = dirty_data.copy()
+    for sensitive in cfg.CONF.action_logging.sensitive_headers:
+        if sensitive in clean_data:
+            del clean_data[sensitive]
+    return clean_data
+
+
+def prepare_request_body_log(body):
+    if not cfg.CONF.action_logging.hide_request_body:
+        return body
+    return 'Request body hidden due to action_logging configuration.'
+
+
+def prepare_response_body_log(body):
+    if not cfg.CONF.action_logging.hide_response_body:
+        return body
+    return 'Response body hidden due to action_logging configuration.'
